//reference: 	http://www.learnopengl.com/#!Lighting/Basic-Lighting
//				http://www.learnopengl.com/#!Lighting/Materials
#version 330 core

struct Material {
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
	float shininess;
};

struct Light {
    vec3 position;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

in vec3 Normal;
in vec3 FragPos;
in vec4 FragPosLightSpace;

out vec4 color;

uniform vec3 viewPos;
uniform mat4 view;
uniform Material material;
uniform Light light;
uniform sampler2D shadowMap;

float ShadowCalculation(vec4 fragPosLightSpace)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    projCoords = projCoords * 0.5f + 0.5f;//transform from [-1,1] to [0,1]
	float closestDepth = texture(shadowMap, projCoords.xy).r; 
	float currentDepth = projCoords.z;
	float shadow = 0.0f;
	
	vec3 norm = normalize(Normal);
	vec3 lightDir = normalize(light.position - FragPos);
	float bias = max(0.05 * (1.0 - dot(norm, lightDir)), 0.005);
	
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;        
        }    
    }
    shadow /= 9.0;
	
	if( projCoords.z > 1.0) {	//should be a > but this works with the rod, why ??
		shadow = 0.0f;
	}
	
	return shadow;	
}

void main()
{
	//ambient lighting
	vec3 ambient = light.ambient * material.ambient;
	vec3 viewPos2 = vec3(view[3].x, view[3].y, view[3].z);
	//diffuse lighting
	//using unit vectors for diffuse lighting
	vec3 norm = normalize(Normal);
	vec3 lightDir = normalize(light.position - FragPos);
	//vec3 lightDir = normalize(-light.position);// - FragPos);
	float diff = max(dot(norm, lightDir), 0.0);
	vec3 diffuse = light.diffuse * (diff * material.diffuse);
	
	//specular lighting
	vec3 viewDir = normalize(viewPos2 - FragPos);
	//vec3 viewDir = normalize(- FragPos);
    vec3 reflectDir = reflect(-lightDir, norm);  
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);	//32 = shininess of highlight ; the larger the number the smaller the spot
    vec3 specular = light.specular * spec * material.specular;  
	
	
	//vec3 result = (ambient + diffuse + specular);
	float shadow = ShadowCalculation(FragPosLightSpace);	
	vec3 lighting = (ambient + (1.0 - shadow) * (diffuse + specular));
	color = vec4(lighting.rgb, 1.0f);
	//color = vec4(result, 1.0f);
}