//reference: http://learnopengl.com/#!Advanced-OpenGL/Shadows/Shadow-Mapping
#version 330 core

//layout(location = 0) out float fragmentdepth;
struct Material {
	sampler2D texture_diffuse1;
	sampler2D texture_specular1;
	float shininess;
};

struct Light {
    vec3 position;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

in vec3 FragPos;
in vec3 Normal;
in vec2 TexCoords;
in vec4 FragPosLightSpace;

out vec4 fragColor;

uniform bool blinnSwitch;
uniform sampler2D shadowMap;
uniform mat4 view;
uniform Light light;
uniform Material material;

float ShadowCalculation(vec4 fragPosLightSpace)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    projCoords = projCoords * 0.5f + 0.5f;//transform from [-1,1] to [0,1]
	float closestDepth = texture(shadowMap, projCoords.xy).r; 
	float currentDepth = projCoords.z;
	float shadow = 0.0f;
	
	vec3 norm = normalize(Normal);
	vec3 lightDir = normalize(light.position - FragPos);
	float bias = max(0.05 * (1.0 - dot(norm, lightDir)), 0.005);
	
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;        
        }    
    }
    shadow /= 9.0;
	
	if( projCoords.z > 1.0) {	//should be a > but this works with the rod, why ??
		shadow = 0.0f;
	}
	
	return shadow;	
}

void main()
{
	//just like blinnphong
	vec3 color = vec3(texture(material.texture_diffuse1, TexCoords));
	vec3 viewPos = vec3(view[3].x, view[3].y, view[3].z);
	vec3 ambient = light.ambient * color;
	
	vec3 viewDir = normalize(viewPos - FragPos);
	vec3 norm = normalize(Normal);
	vec3 lightDir = normalize(light.position - FragPos);
	
	float diff = max(dot(norm, lightDir), 0.0);	
	vec3 diffuse = light.diffuse * diff;
	
	float spec = 0.0;
	
	if(blinnSwitch) {
		vec3 halfwayDir = normalize(lightDir + viewDir);
		spec = pow(max(dot(norm, halfwayDir), 0.0), material.shininess);
	}
	else {
		vec3 reflectDir = reflect(-lightDir, norm);  
		spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
	}
	
	vec3 specular = light.specular * spec;
	
	//shadow
	float shadow = ShadowCalculation(FragPosLightSpace);	
	vec3 lighting = (ambient + (1.0 - shadow) * (diffuse + specular)) * color;
	//vec3 lighting = (ambient + diffuse + specular) * color;
	fragColor = vec4(lighting.bgr,1.0f);
}